searchData={"items":[{"type":"module","title":"Lissome","doc":"Integration of Gleam's Lustre framework with Phoenix LiveView","ref":"Lissome.html"},{"type":"module","title":"Lissome.Component","doc":"","ref":"Lissome.Component.html"},{"type":"function","title":"Lissome.Component.lustre/1","doc":"Renders a lustre app.","ref":"Lissome.Component.html#lustre/1"},{"type":"function","title":"Attributes - Lissome.Component.lustre/1","doc":"* `name` (`:string`) (required) - The name of the Gleam module to render relative to the src directory.Examples include `\"my_lustre_app\"`, and `\"pages/home\"`.\n* `flags` (`:map`) - Initial values to pass to the Gleam module. Defaults to `%{}`.\n* `init_fn` (`:string`) - the name of your Gleam function that initializes the model. Defaults to `\"init\"`.\n* `view_fn` (`:string`) - the name of your Gleam function that renders the view. Defaults to `\"view\"`.\n* `id` (`:string`) - The id Lustre targets to render into. Defaults to `\"app\"`.\n* `class` (`:string`) - The class name to apply to the rendered app. Defaults to `\"\"`.\n* `ssr` (`:boolean`) - Whether to render the app on the server side. Defaults to `true`.","ref":"Lissome.Component.html#lustre/1-attributes"},{"type":"module","title":"Lissome.Render","doc":"","ref":"Lissome.Render.html"},{"type":"function","title":"Lissome.Render.render_lustre/3","doc":"Renders a lustre app in client side.\n\nThis function just renders the root container and the script tags necessary to mount the app.","ref":"Lissome.Render.html#render_lustre/3"},{"type":"function","title":"Lissome.Render.ssr_lustre/5","doc":"Renders a lustre app in server side.\n\nThis function will call the `init_fn` function to get the initial model and then the `view_fn` function to get the initial view.","ref":"Lissome.Render.html#ssr_lustre/5"},{"type":"module","title":"Lissome.Utils","doc":"","ref":"Lissome.Utils.html"},{"type":"function","title":"Lissome.Utils.format_module_name/1","doc":"Turns a module name into a valid Gleam module name.\n\nGleam uses `@` to separate paths in module names. For example, a module\nin the directory `src/pages/home` would be compiled as `pages@home`. We process the path to turn it into the correct erlang module name that we can call in Elixir.","ref":"Lissome.Utils.html#format_module_name/1"},{"type":"function","title":"Examples: - Lissome.Utils.format_module_name/1","doc":"iex> Lissome.Utils.format_module_name(\"nested/nested/nested/mod\")\n  :nested@nested@nested@mod\n\n  iex> Lissome.Utils.format_module_name(\"home\")\n  :home","ref":"Lissome.Utils.html#format_module_name/1-examples"},{"type":"function","title":"Lissome.Utils.json/1","doc":"","ref":"Lissome.Utils.html#json/1"},{"type":"task","title":"mix compile.gleam_js","doc":"Compile Gleam source files to JavaScript and then bundle them using esbuild binary\n\nBuilt files are placed in the `priv/static/gleam` folder.","ref":"Mix.Tasks.Compile.GleamJs.html"},{"type":"task","title":"Examples: - mix compile.gleam_js","doc":"# Compile Gleam to Javascript in a Mix project\n    # and bundle with esbuild in minified mode\n    mix compile.gleam_js --minify\n\nGleam compilation will not occur where no `.gleam` files are located.\n\nTo use this taks, first make sure you have the Gleam to Erlang compiler `gleam` from `mix_gleam` in your compilers list, then include this task in your project's `mix.exs` with, e.g.:\n\n    def project do\n      [\n        compilers: [:gleam, :gleam_js] ++ Mix.compilers(),\n      ]\n    end\n\nCredit goes to the [mix_gleam](https://github.com/gleam-lang/mix_gleam) project, on which this compiler has been modeled.","ref":"Mix.Tasks.Compile.GleamJs.html#module-examples"},{"type":"extras","title":"Lissome","doc":"# Lissome\n\nLissome is a library to integrate the [Gleam](https://gleam.run/) frontend framework [Lustre](https://hexdocs.pm/lustre/lustre.html) with Phoenix Live View.\n\n> [!WARNING]\n> This project is on early stage of development and breaking changes are expected.","ref":"readme.html"},{"type":"extras","title":"Setup - Lissome","doc":"First, make sure you have the Gleam compiler installed. Instructions can be found [here](https://gleam.run/getting-started/installing/)\n\n1. We will use a tool called `mix_gleam` to manage a Gleam project with Mix. Follow the [instructions](https://github.com/gleam-lang/mix_gleam?tab=readme-ov-file#installation) to setup it.\n\n2. Add `lissome` to your `mix.exs` file:\n\n```elixir\ndef deps do\n  [\n    ...,\n    {:lissome, \"~> 0.2.0\"},\n  ]\nend\n```\n3. Create a `gleam.toml` file with your Gleam dependencies:\n\n```toml\nname = \"your_app_name\"\n\n[dependencies]\ngleam_stdlib = \">= 0.44.0 and < 2.0.0\"\nlustre = \">= 4.6.3 and < 5.0.0\"\n\n[dev-dependencies]\ngleeunit = \">= 1.0.0 and < 2.0.0\"\n```\n\n4. Run:\n\n```bash\nmix deps.get\ngleam deps download\n```\n\n> [!NOTE]\n> Although `mix_gleam` is able to install Gleam dependencies, it doesn't manages dependencies well outside Erlang target. For that reason, we usea  `gleam.toml` file and the Gleam tooling to manage Gleam dependencies.","ref":"readme.html#setup"},{"type":"extras","title":"Usage - Lissome","doc":"To render a Lustre app, we need to define a Gleam module that contains a public `main` function. This function must start a Lustre app created with the `lustre.simple` function.\n\n```gleam\n//// src/hello.gleam\n\npub fn init() {\n  ...\n}\n\npub fn update(msg, model) {\n  ...\n}\n\npub fn view(model) {\n  ...\n}\n\npub fn main() {\n  let app = lustre.simple(init, update, view)\n  let assert Ok(_) = lustre.start(app, \"#app\", Nil)\n\n  Nil\n}\n```\n\nNow, inside `HEEX` we can render it using the `lustre` component:\n\n```elixir\ndefmodule MyApp.MyLiveView do\n  use MyApp, :live_view\n\n  import Lissome.Component\n\n  def mount(_params, _session, socket) do\n    {:ok, socket}\n  end\n\n  def render(assigns) do\n    ~H\"\"\"\n     \n       Content rendered with Phoenix Live View \n       \n        <.lustre id=\"app\" name=\"hello\" />\n       \n     \n    \"\"\"\n  end\nend\n```\n\nCheck out the project in the `example` directory for a complete code example.","ref":"readme.html#usage"},{"type":"extras","title":"SSR - Lissome","doc":"Thanks to the ability of Gleam to compile to both Erlang and JavaScript, we can do server-side rendering of Lustre without having to install Node.js. This is why `Lissome` has SSR enabled by default, but you can disable it by passing `ssr={false}` to the `lustre` component.\n\nKeep in mind that `Lissome` will call the `init` and the `view` functions of your Gleam module in order to render the initial HTML. By default `Lissome` will look for functions with that name in your module. If you happen to named them differently, you can pass to the `lustre` component `init_fn` with the name of your function responsible for initializing the model and `view_fn` with the name of your function responsible for rendering the view. Also, both functions must be public.\n\n```elixir\n<.lustre\n  id=\"app\"\n  name=\"hello\"\n  init_fn=\"my_init_function\"\n  view_fn=\"my_view_function\"\n/>\n```","ref":"readme.html#ssr"},{"type":"extras","title":"Use cases - Lissome","doc":"In my experience, many part of the UI in Phoenix apps could be done with `Phoenix LiveView`; even for parts that are heavily interactive we have ways to build components like modals using JS commands and [perform optimistic updates](https://hexdocs.pm/phoenix_live_view/syncing-changes.html) by combining JS commands and Tailwind CSS classes.\n\nHowever, there are still parts of the UI that are very complex to implement in `Phoenix LiveView`, often because they have a non-trivial client-side state.\n\nIn that situations, we could use Gleam and its fronted framework, Lustre, as they are well suited to handle that kind of state in a simple way. Also, Gleam has interopability with Elixir and both shares many concepts, such as immutability, functional paradigms and pattern matching.\n\n`Lissome` is designed to address *only* the gaps of `Phoenix LiveView` when it comes to client-side state.","ref":"readme.html#use-cases"},{"type":"extras","title":"Roadmap - Lissome","doc":"- [ ] Improvements to the SSR.\n- [ ] Gleam's helpers for communicating with Phoenix LiveView and supporting Lustre's effects.\n- [ ] Hot module replacement for Gleam.\n- [ ] Support for [`LiveJson`](https://github.com/Miserlou/live_json).\n- [ ] Support for Lustre's `server components`.","ref":"readme.html#roadmap"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.1"}}